#include <Demo/Scene.h>
#include <Render/D3DDrawUtil.h>

Scene* Scene::m_pInstance = NULL;

void Scene::CreateInstance()
{
	m_pInstance = new Scene();
}

Scene* Scene::Instance()
{
	return m_pInstance;
}
void Scene::DestoryInstance()
{
	if(!m_pInstance)
		return;

	delete m_pInstance;
	m_pInstance = NULL;
}

Scene::Scene()
{
	CreateWorld();
	CreateGround();
}

Scene::~Scene()
{
	ReleaseRigidBodies();
	ReleaseGround();
	ReleaseWorld();
}

void Scene::Update(const float detaTime)
{
	m_dynamicsWorld->stepSimulation(detaTime);
}

void Scene::Render()
{
	D3DDrawUtil *debugRender = D3DDrawUtil::Instance();

	debugRender->DrawGrid(30.0f, 30.0f, 20, 20, D3DCOLOR_COLORVALUE(1.0f, 1.0f, 1.0f, .2f));

	for(int i=0; i<(int)m_grounds.size(); i++)
	{
		debugRender->DrawWireRigidBody(m_grounds[i], 0xFF0000FF);
	}

	for(int i=0; i<(int)m_rigidBodies.size(); i++)
	{
		debugRender->DrawWireRigidBody(m_rigidBodies[i], 0xFF000000);
	}
}

void Scene::CreateWorld()
{
	m_collisionConfiguration = new btDefaultCollisionConfiguration();
	
	m_dispatcher = new btCollisionDispatcher(m_collisionConfiguration);

	m_broadphase = new btDbvtBroadphase();

	m_solver = new btSequentialImpulseConstraintSolver;

	m_dynamicsWorld = new btDiscreteDynamicsWorld(m_dispatcher, m_broadphase, m_solver, m_collisionConfiguration);
	m_dynamicsWorld->setGravity(btVector3(0.0f, 0.0f, -9.8f));
}

void Scene::ReleaseWorld()
{
	if(m_dynamicsWorld)
	{
		delete m_dynamicsWorld;
		m_dynamicsWorld = NULL;
	}

	if(m_solver)
	{
		delete m_solver;
		m_solver = NULL;
	}

	if(m_broadphase)
	{
		delete m_broadphase;
		m_broadphase = NULL;
	}

	if(m_dispatcher)
	{
		delete m_dispatcher;
		m_dispatcher = NULL;
	}

	if(m_collisionConfiguration)
	{
		delete m_collisionConfiguration;
		m_collisionConfiguration = NULL;
	}
}

void Scene::CreateGround()
{
	const float width = 40.0f;
	const float depth = 1.0f;

	{
		const btVector3 dimension = btVector3(6.0f, 10.0f, 1.0f);
		const float angle = 3.141592f*0.1f;

		btTransform hingeTM;
		hingeTM.setIdentity();
		hingeTM.setOrigin(btVector3(0.0f, dimension.m_floats[1], dimension.m_floats[2]));

		btTransform rotationTM;
		btQuaternion q(btVector3(1.0f, 0.0f, 0.0f), -angle);
		rotationTM.setIdentity();
		rotationTM.setRotation(q);

		btTransform localTM;
		localTM.setIdentity();
		localTM.setOrigin(btVector3(0.0f, -dimension.m_floats[1], 0.0f));

		btTransform bodyTM = hingeTM*rotationTM*localTM;

		// Box 
		btCollisionShape* boxShape = new btBoxShape(dimension);

		// IneritaTensor 
		btVector3 localInertia(0,0,0);

		btDefaultMotionState* myMotionState = new btDefaultMotionState(bodyTM);
		btRigidBody::btRigidBodyConstructionInfo rbInfo(0.0f, myMotionState, boxShape, localInertia);
		rbInfo.m_restitution = 1.0f;

		btRigidBody *ground = new btRigidBody(rbInfo);

		m_dynamicsWorld->addRigidBody(ground);
		m_grounds.push_back(ground);
	}

	{
		btTransform bodyTM;
		bodyTM.setIdentity();
		bodyTM.setOrigin(btVector3(0.0f, 0.0f, -depth));

		// Box 
		btCollisionShape* boxShape = new btBoxShape(btVector3(width, width, depth));

		// IneritaTensor 
		btVector3 localInertia(0,0,0);

		btDefaultMotionState* myMotionState = new btDefaultMotionState(bodyTM);
		btRigidBody::btRigidBodyConstructionInfo rbInfo(0.0f, myMotionState, boxShape, localInertia);
		rbInfo.m_restitution = 1.0f;

		btRigidBody *ground = new btRigidBody(rbInfo);

		m_dynamicsWorld->addRigidBody(ground);
		m_grounds.push_back(ground);
	}


	{
		btTransform bodyTM;
		bodyTM.setIdentity();
		bodyTM.setOrigin(btVector3(0.0f, width + depth, 0.0f));

		// Box 
		btCollisionShape* boxShape = new btBoxShape(btVector3(width, depth, 2.0f*depth));

		// IneritaTensor 
		btVector3 localInertia(0,0,0);

		btDefaultMotionState* myMotionState = new btDefaultMotionState(bodyTM);
		btRigidBody::btRigidBodyConstructionInfo rbInfo(0.0f, myMotionState, boxShape, localInertia);
		rbInfo.m_restitution = 1.0f;

		btRigidBody *ground = new btRigidBody(rbInfo);

		m_dynamicsWorld->addRigidBody(ground);
		m_grounds.push_back(ground);
	}
}

void Scene::ReleaseGround()
{
	for(int i=0; i<(int)m_grounds.size(); i++)
	{
		m_dynamicsWorld->removeRigidBody(m_grounds[i]);

		delete m_grounds[i]->getCollisionShape();
		delete m_grounds[i]->getMotionState();
		delete m_grounds[i];
	}

	m_grounds.clear();
}

void Scene::CreateRigidBodies()
{
	{
		// 1 
		const float mass = 1.0f;

		//RidigBody (0, 0, 10) 
		btTransform bodyTM;
		bodyTM.setIdentity();
		bodyTM.setOrigin(btVector3(-4.0f, 0.0f, 20.0f));

		// Box 
		btCollisionShape* boxShape = new btBoxShape(btVector3(1.0f, 1.0f, 1.0f));

		// IneritaTensor 
		btVector3 localInertia(0,0,0);
		boxShape->calculateLocalInertia(mass, localInertia);

		btDefaultMotionState* myMotionState = new btDefaultMotionState(bodyTM);
		btRigidBody::btRigidBodyConstructionInfo rbInfo(mass, myMotionState, boxShape, localInertia);
		rbInfo.m_restitution = m_physicsProperty.m_restitution;
		rbInfo.m_friction = m_physicsProperty.m_friction;
		rbInfo.m_linearDamping = m_physicsProperty.m_linearDamping;
		rbInfo.m_angularDamping = m_physicsProperty.m_angularDamping;
		btRigidBody* body = new btRigidBody(rbInfo);

		m_dynamicsWorld->addRigidBody(body);
		m_rigidBodies.push_back(body);
	}

	{
		// 1 
		const float mass = 1.0f;

		//RidigBody (0, 0, 10) 
		btTransform bodyTM;
		bodyTM.setIdentity();
		bodyTM.setOrigin(btVector3(-2.0f, 0.0f, 20.0f));

		// Sphere 
		btCollisionShape* sphereShape = new btSphereShape(1.0f);

		// IneritaTensor 
		btVector3 localInertia(0,0,0);
		sphereShape->calculateLocalInertia(mass, localInertia);

		btDefaultMotionState* myMotionState = new btDefaultMotionState(bodyTM);
		btRigidBody::btRigidBodyConstructionInfo rbInfo(mass, myMotionState, sphereShape, localInertia);
		rbInfo.m_restitution = m_physicsProperty.m_restitution;
		rbInfo.m_friction = m_physicsProperty.m_friction;
		rbInfo.m_linearDamping = m_physicsProperty.m_linearDamping;
		rbInfo.m_angularDamping = m_physicsProperty.m_angularDamping;
		btRigidBody* body = new btRigidBody(rbInfo);

		m_dynamicsWorld->addRigidBody(body);
		m_rigidBodies.push_back(body);
	}

	{
		// 1 
		const float mass = 1.0f;

		//RidigBody (0, 0, 10) 
		btTransform bodyTM;
		bodyTM.setIdentity();
		bodyTM.setOrigin(btVector3(2.0f, 0.0f, 20.0f));

		// Capsule 
		btCollisionShape* capsuleShape = new btCapsuleShapeZ(0.5f, 2.0f);

		// IneritaTensor 
		btVector3 localInertia(0,0,0);
		capsuleShape->calculateLocalInertia(mass, localInertia);

		btDefaultMotionState* myMotionState = new btDefaultMotionState(bodyTM);
		btRigidBody::btRigidBodyConstructionInfo rbInfo(mass, myMotionState, capsuleShape, localInertia);
		rbInfo.m_restitution = m_physicsProperty.m_restitution;
		rbInfo.m_friction = m_physicsProperty.m_friction;
		rbInfo.m_linearDamping = m_physicsProperty.m_linearDamping;
		rbInfo.m_angularDamping = m_physicsProperty.m_angularDamping;
		btRigidBody* body = new btRigidBody(rbInfo);

		m_dynamicsWorld->addRigidBody(body);
		m_rigidBodies.push_back(body);
	}

	{
		// 1 
		const float mass = 1.0f;

		//RidigBody (0, 0, 10) 
		btTransform bodyTM;
		bodyTM.setIdentity();
		bodyTM.setOrigin(btVector3(4.0f, 0.0f, 20.0f));

		// Cylinder 
		btCollisionShape* cylinderShape = new btCylinderShapeZ(btVector3(0.5f, 0.5f, 1.0f));

		// IneritaTensor 
		btVector3 localInertia(0,0,0);
		cylinderShape->calculateLocalInertia(mass, localInertia);

		btDefaultMotionState* myMotionState = new btDefaultMotionState(bodyTM);
		btRigidBody::btRigidBodyConstructionInfo rbInfo(mass, myMotionState, cylinderShape, localInertia);
		rbInfo.m_restitution = m_physicsProperty.m_restitution;
		rbInfo.m_friction = m_physicsProperty.m_friction;
		rbInfo.m_linearDamping = m_physicsProperty.m_linearDamping;
		rbInfo.m_angularDamping = m_physicsProperty.m_angularDamping;
		btRigidBody* body = new btRigidBody(rbInfo);

		m_dynamicsWorld->addRigidBody(body);
		m_rigidBodies.push_back(body);
	}
}

void Scene::ReleaseRigidBodies()
{
	for(int i=0; i<(int)m_rigidBodies.size(); i++)
	{
		m_dynamicsWorld->removeRigidBody(m_rigidBodies[i]);

		delete m_rigidBodies[i]->getCollisionShape();
		delete m_rigidBodies[i]->getMotionState();
		delete m_rigidBodies[i];
	}

	m_rigidBodies.clear();
}

Scene::PhysicsProperty Scene::GetPhysicsProperty()
{
	return m_physicsProperty;
}

void Scene::SetPhysicsProperty(PhysicsProperty &property)
{
	m_physicsProperty = property;

	UpdatePhysicsProperty();
}

void Scene::UpdatePhysicsProperty()
{
	for(int i=0; i<(int)m_rigidBodies.size(); i++)
	{
		m_rigidBodies[i]->setFriction(m_physicsProperty.m_friction);
		m_rigidBodies[i]->setRestitution(m_physicsProperty.m_restitution);
		m_rigidBodies[i]->setDamping(m_physicsProperty.m_linearDamping, m_physicsProperty.m_angularDamping);
	}
}

