#ifndef SCENE_H
#define SCENE_H

#include<btBulletDynamicsCommon.h>
#include<vector>

class Scene
{
public:
	struct PhysicsProperty
	{
		float m_restitution;
		float m_friction;
		float m_linearDamping;
		float m_angularDamping;

		PhysicsProperty()
		{
			m_restitution = 0.0f;
			m_friction = 0.5f;
			m_linearDamping = 0.0f;
			m_angularDamping = 0.0f;
		}
	};

	//
	Scene(void);
	
private:
	static Scene *m_pInstance;

public:
	//
	static void CreateInstance();
	
	//
	static Scene* Instance();

	//
	static void DestoryInstance();

	//
	void Update(const float detaTime);

	//
	void Render();

	//
	void CreateRigidBodies();

	//
	void ReleaseRigidBodies();

	//
	PhysicsProperty GetPhysicsProperty();

	//
	void SetPhysicsProperty(PhysicsProperty &property);

protected:
	//
	virtual ~Scene(void);

	//
	void CreateWorld();

	//
	void ReleaseWorld();

	//
	void CreateGround();

	//
	void ReleaseGround();

	//
	void UpdatePhysicsProperty();

private:
	btBroadphaseInterface*	m_broadphase;
	btCollisionDispatcher*	m_dispatcher;
	btConstraintSolver* m_solver;
	btDefaultCollisionConfiguration* m_collisionConfiguration;
	btDynamicsWorld* m_dynamicsWorld;
	
	std::vector<btRigidBody*> m_rigidBodies;
	std::vector<btRigidBody*> m_grounds;

	PhysicsProperty m_physicsProperty;
};

#endif
