#include <Panel/PropertyPanel.h>
#include <Demo/Scene.h>

BEGIN_EVENT_TABLE(PropertyPanel, wxPanel)

EVT_SIZE(OnSize)
EVT_BUTTON(ID_BUTTON_CREATERIGIDBODIES, OnButtonCreateRigidBodies)
EVT_BUTTON(ID_BUTTON_RELEASERIGIDBODIES, OnButtonReleaseRigidBodies)
EVT_SLIDER(ID_SLIDER_RESTITUTION, OnSliderRestitution)
EVT_SLIDER(ID_SLIDER_FRICTION, OnSliderFriction)
EVT_SLIDER(ID_SLIDER_LINEARDAMPING, OnSliderLinearDamping)
EVT_SLIDER(ID_SLIDER_ANGULARDAMPING, OnSliderAngularDamping)

END_EVENT_TABLE()

PropertyPanel::PropertyPanel(wxWindow *parent, const wxPoint& pos, const wxSize& size):
wxPanel(parent, -1, pos, size)
{
	CreateControl();
}

PropertyPanel::~PropertyPanel(void)
{
}

void PropertyPanel::CreateControl()
{
	wxBoxSizer *pSizer = new wxBoxSizer(wxVERTICAL);
	SetSizer(pSizer);

	m_pButtonCreateRigidBodies = new wxButton(this, ID_BUTTON_CREATERIGIDBODIES, wxT("Create"));
	pSizer->Add(m_pButtonCreateRigidBodies, 0, wxEXPAND|wxALL, 2);

	m_pButtonReleaseRigidBodies = new wxButton(this, ID_BUTTON_RELEASERIGIDBODIES, wxT("Release"));
	pSizer->Add(m_pButtonReleaseRigidBodies, 0, wxEXPAND|wxALL, 2);

	pSizer->AddSpacer(10);

	wxStaticBoxSizer *pSizerRigidBody = new wxStaticBoxSizer(wxVERTICAL, this, wxT("Property"));
	pSizer->Add(pSizerRigidBody, 0, wxEXPAND|wxALL, 2);
	{
		wxBoxSizer *pSizerRestitution = new wxBoxSizer(wxHORIZONTAL);
		pSizerRigidBody->Add(pSizerRestitution, 0, wxEXPAND|wxALL, 2);
		{
			wxStaticText *pText = new wxStaticText(this, wxID_ANY, wxT("Restitution"), wxDefaultPosition, wxSize(60, -1));
			pSizerRestitution->Add(pText, 0, wxEXPAND|wxALL, 2);

			m_pTextRestitution = new wxTextCtrl(this, ID_TEXT_RESTITUTION, wxT(""), wxDefaultPosition, wxSize(40, -1));
			pSizerRestitution->Add(m_pTextRestitution, 0, wxEXPAND|wxALL, 2);
			m_pTextRestitution->Disable();

			m_pSliderRestitution = new wxSlider(this, ID_SLIDER_RESTITUTION, 0, 0, 100);
			pSizerRestitution->Add(m_pSliderRestitution, 1, wxEXPAND|wxALL, 2);
		}

		wxBoxSizer *pSizerFriction = new wxBoxSizer(wxHORIZONTAL);
		pSizerRigidBody->Add(pSizerFriction, 0, wxEXPAND|wxALL, 2);
		{
			wxStaticText *pText = new wxStaticText(this, wxID_ANY, wxT("Friction"), wxDefaultPosition, wxSize(60, -1));
			pSizerFriction->Add(pText, 0, wxEXPAND|wxALL, 2);

			m_pTextFriction = new wxTextCtrl(this, ID_TEXT_FRICTION, wxT(""), wxDefaultPosition, wxSize(40, -1));
			pSizerFriction->Add(m_pTextFriction, 0, wxEXPAND|wxALL, 2);
			m_pTextFriction->Disable();

			m_pSliderFriction = new wxSlider(this, ID_SLIDER_FRICTION, 0, 0, 100);
			pSizerFriction->Add(m_pSliderFriction, 1, wxEXPAND|wxALL, 2);
		}

		wxBoxSizer *pSizerLinearDamping = new wxBoxSizer(wxHORIZONTAL);
		pSizerRigidBody->Add(pSizerLinearDamping, 0, wxEXPAND|wxALL, 2);
		{
			wxStaticText *pText = new wxStaticText(this, wxID_ANY, wxT("LDamping"), wxDefaultPosition, wxSize(60, -1));
			pSizerLinearDamping->Add(pText, 0, wxEXPAND|wxALL, 2);

			m_pTextLinearDamping = new wxTextCtrl(this, ID_TEXT_LINEARDAMPING, wxT(""), wxDefaultPosition, wxSize(40, -1));
			pSizerLinearDamping->Add(m_pTextLinearDamping, 0, wxEXPAND|wxALL, 2);
			m_pTextLinearDamping->Disable();

			m_pSliderLinearDamping = new wxSlider(this, ID_SLIDER_LINEARDAMPING, 0, 0, 100);
			pSizerLinearDamping->Add(m_pSliderLinearDamping, 1, wxEXPAND|wxALL, 2);
		}

		wxBoxSizer *pSizerAngularDamping = new wxBoxSizer(wxHORIZONTAL);
		pSizerRigidBody->Add(pSizerAngularDamping, 0, wxEXPAND|wxALL, 2);
		{
			wxStaticText *pText = new wxStaticText(this, wxID_ANY, wxT("ADamping"), wxDefaultPosition, wxSize(60, -1));
			pSizerAngularDamping->Add(pText, 0, wxEXPAND|wxALL, 2);

			m_pTextAngularDamping = new wxTextCtrl(this, ID_TEXT_ANGULARDAMPING, wxT(""), wxDefaultPosition, wxSize(40, -1));
			pSizerAngularDamping->Add(m_pTextAngularDamping, 0, wxEXPAND|wxALL, 2);
			m_pTextAngularDamping->Disable();

			m_pSliderAngularDamping = new wxSlider(this, ID_SLIDER_ANGULARDAMPING, 0, 0, 100);
			pSizerAngularDamping->Add(m_pSliderAngularDamping, 1, wxEXPAND|wxALL, 2);
		}
	}

	pSizer->AddSpacer(10);
}

void PropertyPanel::UpdatePanel()
{
	Scene::PhysicsProperty &property = Scene::Instance()->GetPhysicsProperty();

	m_pTextRestitution->SetValue(wxString::Format("%.3f", property.m_restitution));
	m_pSliderRestitution->SetValue((int)(property.m_restitution*100.0f));
	m_pTextFriction->SetValue(wxString::Format("%.3f", property.m_friction));
	m_pSliderFriction->SetValue((int)(property.m_friction*100.0f));
	m_pTextLinearDamping->SetValue(wxString::Format("%.3f", property.m_linearDamping));
	m_pSliderLinearDamping->SetValue((int)(property.m_linearDamping*100.0f));
	m_pTextAngularDamping->SetValue(wxString::Format("%.3f", property.m_angularDamping));
	m_pSliderAngularDamping->SetValue((int)(property.m_angularDamping*100.0f));
}

void PropertyPanel::OnSize(wxSizeEvent& event)
{
	Refresh();

	event.Skip();
}

void PropertyPanel::OnButtonCreateRigidBodies(wxCommandEvent& event)
{
	Scene::Instance()->CreateRigidBodies();
}

void PropertyPanel::OnButtonReleaseRigidBodies(wxCommandEvent& event)
{
	Scene::Instance()->ReleaseRigidBodies();
}

void PropertyPanel::OnSliderRestitution(wxCommandEvent& event)
{
	Scene::PhysicsProperty property = Scene::Instance()->GetPhysicsProperty();
	property.m_restitution = 0.01f*m_pSliderRestitution->GetValue();
	
	Scene::Instance()->SetPhysicsProperty(property);

	m_pTextRestitution->SetValue(wxString::Format("%.3f", property.m_restitution));
}

void PropertyPanel::OnSliderFriction(wxCommandEvent& event)
{
	Scene::PhysicsProperty property = Scene::Instance()->GetPhysicsProperty();
	property.m_friction = 0.01f*m_pSliderFriction->GetValue();

	Scene::Instance()->SetPhysicsProperty(property);

	m_pTextFriction->SetValue(wxString::Format("%.3f", property.m_friction));
}

void PropertyPanel::OnSliderLinearDamping(wxCommandEvent& event)
{
	Scene::PhysicsProperty property = Scene::Instance()->GetPhysicsProperty();
	property.m_linearDamping = 0.01f*m_pSliderLinearDamping->GetValue();

	Scene::Instance()->SetPhysicsProperty(property);
	
	m_pTextLinearDamping->SetValue(wxString::Format("%.3f", property.m_linearDamping));
}

void PropertyPanel::OnSliderAngularDamping(wxCommandEvent& event)
{
	Scene::PhysicsProperty property = Scene::Instance()->GetPhysicsProperty();
	property.m_angularDamping = 0.01f*m_pSliderAngularDamping->GetValue();

	Scene::Instance()->SetPhysicsProperty(property);

	m_pTextAngularDamping->SetValue(wxString::Format("%.3f", property.m_angularDamping));
}

