#include <Panel/RenderWindow.h>
#include <Demo/Scene.h>
#include <Render/D3DDrawUtil.h>
#include <Time/Timer.h>

BEGIN_EVENT_TABLE(RenderWindow, wxWindow)

EVT_ERASE_BACKGROUND (RenderWindow::OnDraw)
EVT_SIZE(RenderWindow::OnSize)
EVT_IDLE (RenderWindow::OnIdle)
EVT_MOUSE_EVENTS(RenderWindow::OnMouse)
EVT_MOUSEWHEEL(RenderWindow::OnMouseWheel)
EVT_LEFT_DCLICK(RenderWindow::OnMouseLDClick)
EVT_ENTER_WINDOW(RenderWindow::OnEnterWindow)
EVT_LEFT_DOWN(RenderWindow::OnLButtonDown)
EVT_LEFT_UP(RenderWindow::OnLButtonUp)
EVT_CHAR(RenderWindow::OnChar)

END_EVENT_TABLE()

RenderWindow::RenderWindow(wxWindow *parent, const wxPoint& pos, const wxSize& size): 
wxWindow(parent, -1, pos, size),
m_fSphereThetaAngle(1.1f), 
m_fSpherePhiAngle(7.39f), 
m_fSphereLength(35.0f), 
m_vTranslation(-0.12f, -0.6f, 2.83f),
m_iScreenX(0),
m_iScreenY(0),
m_iScreenPreX(0),
m_iScreenPreY(0)
{

}

RenderWindow::~RenderWindow(void)
{
}

void RenderWindow::OnIdle(wxIdleEvent& event) 
{
	Refresh();
}

void RenderWindow::OnSize(wxSizeEvent& event)
{
	Refresh();
}

void RenderWindow::OnMouseWheel(wxMouseEvent &event)
{
	if(event.AltDown())
		m_fSphereLength -= 0.005f*event.GetWheelRotation();

	if(m_fSphereLength < 2.0f)
		m_fSphereLength = 2.0f;

	event.Skip();
}

void RenderWindow::OnEnterWindow( wxMouseEvent& event)
{
	SetFocus();
}

void RenderWindow::OnMouse(wxMouseEvent& event)
{
	static int iPreX, iPreY;

	int iX = (int)event.GetX();
	int iY = (int)event.GetY();
	int iXStep = iX - iPreX;
	int iYStep = iY - iPreY;

	if(event.LeftIsDown() && event.AltDown())
	{
		if(iXStep*iXStep >= iYStep*iYStep)
		{
			m_fSpherePhiAngle -= 0.01f*iXStep;
		}
		else
		{
			m_fSphereThetaAngle -= 0.01f*iYStep;
		}
	}
	else if(event.RightIsDown() && event.AltDown())
	{
		D3DXMATRIX invViewM;
		D3DXMatrixInverse(&invViewM, NULL, &m_viewM);

		if(iXStep*iXStep >= iYStep*iYStep)
		{
			m_vTranslation -= iXStep*0.05f*D3DXVECTOR3(invViewM._11, invViewM._12, invViewM._13); 
		}
		else
		{
			m_vTranslation += iYStep*0.05f*D3DXVECTOR3(invViewM._21, invViewM._22, invViewM._23);
		}
	}

	iPreX = iX;
	iPreY = iY;

	m_iScreenPreX = m_iScreenX;
	m_iScreenPreY = m_iScreenY;
	m_iScreenX = iX;
	m_iScreenY = iY;

	event.Skip();
}

void RenderWindow::OnMouseLDClick(wxMouseEvent& event)
{

}

void RenderWindow::OnLButtonDown(wxMouseEvent& event)
{

}

void RenderWindow::OnLButtonUp(wxMouseEvent& event)
{

}

void RenderWindow::OnChar(wxKeyEvent& event)
{


}

void RenderWindow::OnDraw(wxEraseEvent& event) 
{
	UpdateMainCamera();
	UpdateRay();
	Update();

	LPDIRECT3DDEVICE9 pd3dDevice = D3DDrawUtil::Instance()->GetDevice();
	D3DVIEWPORT9 BufferViewPort, WindowViewPort;
	pd3dDevice->GetViewport(&BufferViewPort);
	wxRect rect = GetSize();
	RECT windowRect;

	WindowViewPort.X      = rect.x;
	WindowViewPort.Y      = rect.y;
	WindowViewPort.Width  = rect.width;
	WindowViewPort.Height = rect.height;
	WindowViewPort.MinZ   = 0.0f;
	WindowViewPort.MaxZ   = 1.0f;

	windowRect.left = rect.x;
	windowRect.bottom = rect.y + rect.height;
	windowRect.right = rect.x + rect.width;
	windowRect.top = rect.y;


	pd3dDevice->SetViewport(&WindowViewPort);
	pd3dDevice->Clear(0, NULL, D3DCLEAR_TARGET | D3DCLEAR_ZBUFFER, D3DCOLOR_COLORVALUE( 0.35f, 0.53f, 0.7, 1.0f ), 1.0f,0);

	if( SUCCEEDED(pd3dDevice->BeginScene()))
	{
		SetupMainCamera();
		SetupLights();

		Render();

		pd3dDevice->EndScene();
	}

	pd3dDevice->Present(&windowRect, &windowRect, static_cast<HWND>(GetHWND()), NULL);
}

void RenderWindow::UpdateRay()
{
	wxRect rect = GetSize();
	float fRatio = 1.0f*rect.width /rect.height;
}

void RenderWindow::UpdateMainCamera()
{
	D3DXMATRIXA16 invMatCamera, matCamera;
	wxRect rect = GetSize();
	float fRatio = 1.0f*rect.width /rect.height;

	invMatCamera._11 = -sinf(m_fSpherePhiAngle); invMatCamera._12 = cosf(m_fSpherePhiAngle); invMatCamera._13 =0.0f; invMatCamera._14 = 0.0f;
	invMatCamera._21 = -cosf(m_fSphereThetaAngle)*cosf(m_fSpherePhiAngle); invMatCamera._22 = -cosf(m_fSphereThetaAngle)*sinf(m_fSpherePhiAngle); invMatCamera._23 = sinf(m_fSphereThetaAngle); invMatCamera._24 = 0.0f;
	invMatCamera._31 = sinf(m_fSphereThetaAngle)*cosf(m_fSpherePhiAngle); invMatCamera._32 = sinf(m_fSphereThetaAngle)*sinf(m_fSpherePhiAngle); invMatCamera._33 = cosf(m_fSphereThetaAngle); invMatCamera._34 = 0.0f;
	invMatCamera._41 = m_fSphereLength *invMatCamera._31 + m_vTranslation.x;  invMatCamera._42 = m_fSphereLength *invMatCamera._32 + m_vTranslation.y;  invMatCamera._43 = m_fSphereLength*invMatCamera._33 + m_vTranslation.z;  invMatCamera._44 = 1.0f;

	D3DXMatrixInverse(&m_viewM, NULL, &invMatCamera);

	D3DXMatrixPerspectiveFovRH( &m_projectionM, D3DX_PI/4, fRatio, 0.1f, 500.0f);
}


void RenderWindow::SetupMainCamera()
{
	LPDIRECT3DDEVICE9 pd3dDevice = D3DDrawUtil::Instance()->GetDevice();
	D3DXMATRIX invViewM;

	D3DXMatrixInverse(&invViewM, NULL, &m_viewM);

	pd3dDevice->SetTransform(D3DTS_VIEW, &m_viewM);
	pd3dDevice->SetTransform(D3DTS_PROJECTION, &m_projectionM);
	D3DDrawUtil::Instance()->SetViewProjectTM(m_viewM * m_projectionM);
}

void RenderWindow::SetupLights()
{
	LPDIRECT3DDEVICE9 pd3dDevice = D3DDrawUtil::Instance()->GetDevice();
	D3DMATERIAL9  mtrl;
	ZeroMemory( &mtrl, sizeof(D3DMATERIAL9) );
	mtrl.Diffuse.r = mtrl.Ambient.r = 1.0f;
	mtrl.Diffuse.g = mtrl.Ambient.g = 1.0f;
	mtrl.Diffuse.b = mtrl.Ambient.b = 1.0f;
	mtrl.Diffuse.a = mtrl.Ambient.a = 1.0f;
	pd3dDevice->SetMaterial( &mtrl );

	D3DXVECTOR3 vecDir;
	D3DLIGHT9 light;
	ZeroMemory( &light, sizeof(D3DLIGHT9) );

	light.Type = D3DLIGHT_DIRECTIONAL;
	light.Diffuse.r = 1.0f;
	light.Diffuse.g  = 1.0f;
	light.Diffuse.b  = 1.0f;
	vecDir = D3DXVECTOR3( 0, 0, -1 );				
	D3DXVec3Normalize( (D3DXVECTOR3*)&light.Direction, &vecDir );
	light.Range       = 1000.0f;

	pd3dDevice->SetLight(0, &light);
	pd3dDevice->LightEnable( 0, FALSE);
	pd3dDevice->SetRenderState( D3DRS_LIGHTING, FALSE );
	pd3dDevice->SetRenderState( D3DRS_AMBIENT, D3DCOLOR_COLORVALUE(0.5f, 0.5f, 0.5f, 1.0f));
}

void RenderWindow::Update()
{
	if(!Timer::Instance())
		return;

	Timer::Instance()->UpdateDeltaTime();
	const float deltaTime = Timer::Instance()->GetDeltaTime();

	Scene::Instance()->Update(deltaTime);
}

void RenderWindow::Render()
{
	Scene::Instance()->Render();
}
