#include <Render/D3DDrawUtil.h>
#include <assert.h>

//#define DEBUG_VS
//#define DEBUG_PS

#define ARGB(A, R, G, B) ( (int)((A & 0xFF) << 24 | \
								 (R & 0xFF) << 16 | \
								 (G & 0xFF) << 8 | \
								 (B & 0xFF)) )

D3DXMATRIX btTransformToD3DXMATIRX(const btTransform& transform)
{
	btVector3 R = transform.getBasis().getColumn(0);
	btVector3 U = transform.getBasis().getColumn(1);
	btVector3 L = transform.getBasis().getColumn(2);
	btVector3 P = transform.getOrigin();

	D3DXMATRIX matM;
	matM._11 = R.m_floats[0]; matM._12 = R.m_floats[1]; matM._13 = R.m_floats[2]; matM._14 = 0.f;
	matM._21 = U.m_floats[0]; matM._22 = U.m_floats[1]; matM._23 = U.m_floats[2]; matM._24 = 0.f;
	matM._31 = L.m_floats[0]; matM._32 = L.m_floats[1]; matM._33 = L.m_floats[2]; matM._34 = 0.f;
	matM._41 = P.m_floats[0]; matM._42 = P.m_floats[1]; matM._43 = P.m_floats[2]; matM._44 = 1.f;

	return matM;
}

D3DDrawUtil* D3DDrawUtil::m_pInstance = 0;

void D3DDrawUtil::CreateInstance(HWND hWnd)
{
	m_pInstance = new D3DDrawUtil(hWnd);
}

D3DDrawUtil* D3DDrawUtil::Instance()
{
	return m_pInstance;
}

void D3DDrawUtil::DestoryInstance()
{
	if(!m_pInstance)
		return;

	delete m_pInstance;
	m_pInstance = NULL;
}

D3DDrawUtil::D3DDrawUtil(HWND hWnd) 
{	
	if(S_OK == CreateD3D(hWnd))
	{
		CreateMatrixStack();
		CreateFont();
		CreateEffect();
		CreateGeometry();
	}
	else
	{
		assert(false);
	}
}

D3DDrawUtil::~D3DDrawUtil(void)
{
	ReleaseMatrixStack();
	ReleaseFont();
	ReleaseGeometry();
	ReleaseEffect();
	ReleaseD3D();
}

HRESULT D3DDrawUtil::CreateD3D(HWND hWnd)
{
	if(NULL == (m_pD3D = Direct3DCreate9(D3D_SDK_VERSION)))
		return E_FAIL;

	D3DDISPLAYMODE d3ddm;
	if(FAILED(m_pD3D->GetAdapterDisplayMode(D3DADAPTER_DEFAULT, &d3ddm)))
	{
		return E_FAIL;
	}

	D3DCAPS9 d3dCaps;
	if(FAILED(m_pD3D->GetDeviceCaps(D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, &d3dCaps)))
	{
		return E_FAIL;
	}

	DWORD dwBehaviorFlags = 0;
	{
		if( d3dCaps.VertexProcessingCaps != 0)
			dwBehaviorFlags |= D3DCREATE_HARDWARE_VERTEXPROCESSING;
		else
			dwBehaviorFlags |= D3DCREATE_SOFTWARE_VERTEXPROCESSING;
	}

	D3DPRESENT_PARAMETERS d3dpp;
	{
		ZeroMemory(&d3dpp, sizeof(d3dpp));
		d3dpp.BackBufferFormat       = d3ddm.Format;
		d3dpp.SwapEffect             = D3DSWAPEFFECT_DISCARD;
		d3dpp.Windowed               = TRUE;
		d3dpp.EnableAutoDepthStencil = TRUE;
		d3dpp.AutoDepthStencilFormat = D3DFMT_D16;
		d3dpp.PresentationInterval   = D3DPRESENT_INTERVAL_IMMEDIATE;
		d3dpp.BackBufferHeight = d3ddm.Height;
		d3dpp.BackBufferWidth = d3ddm.Width;
	}

#ifdef DEBUG_SHADER 
	if(FAILED(m_pD3D->CreateDevice(D3DADAPTER_DEFAULT, D3DDEVTYPE_REF, hWnd, D3DCREATE_SOFTWARE_VERTEXPROCESSING, &d3dpp, &m_pd3dDevice)))
	{
		return E_FAIL;
	}
#else
	if(FAILED(m_pD3D->CreateDevice(D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, hWnd, dwBehaviorFlags, &d3dpp, &m_pd3dDevice)))
	{
		return E_FAIL;
	}
#endif
	m_pd3dDevice->SetRenderState(D3DRS_LIGHTING, FALSE);
	m_pd3dDevice->SetRenderState(D3DRS_ZENABLE, D3DZB_TRUE);
	m_pd3dDevice->SetRenderState(D3DRS_CULLMODE, D3DCULL_CW); 
	m_pd3dDevice->SetRenderState(D3DRS_SRCBLEND,D3DBLEND_SRCALPHA); 
	m_pd3dDevice->SetRenderState(D3DRS_DESTBLEND, D3DBLEND_INVSRCALPHA);

	return S_OK;
}

void D3DDrawUtil::ReleaseD3D()
{
	if(m_pd3dDevice)
	{
		m_pd3dDevice->Release();
		m_pd3dDevice = NULL;
	}

	if(m_pD3D)
	{
		m_pD3D->Release();
		m_pD3D = NULL;
	}
}

void D3DDrawUtil::CreateMatrixStack()
{
	D3DXCreateMatrixStack(0,&m_pMatrixStack);
	m_pMatrixStack->LoadIdentity();
}

void D3DDrawUtil::ReleaseMatrixStack()
{
	if(m_pMatrixStack)
	{
		m_pMatrixStack->Release();
		m_pMatrixStack = NULL;
	}
}

void D3DDrawUtil::CreateFont()
{
	HRESULT hr;

	hr = D3DXCreateFont( m_pd3dDevice, 22, 0, FW_BOLD, 0, FALSE, 
		DEFAULT_CHARSET, OUT_DEFAULT_PRECIS, DEFAULT_QUALITY, 
		DEFAULT_PITCH | FF_DONTCARE, TEXT("Arial"), 
		&m_pFont );

	if( FAILED( hr ) )
		assert("Call to D3DXCreateFont failed!");
}

void D3DDrawUtil::ReleaseFont()
{
	if(m_pFont)
	{
		m_pFont->Release();
		m_pFont = NULL;
	}
}

void D3DDrawUtil::CreateEffect()
{
	HRESULT hr;
	LPD3DXBUFFER pErr=NULL;
	DWORD dwShaderFlags;

#ifdef DEBUG_SHADER
	dwShaderFlags = 2240;
#else
	dwShaderFlags = 0;
#endif 


	if( FAILED( hr = D3DXCreateEffectFromFile(
		m_pd3dDevice, "shader\\wireframe.fx", NULL, NULL, 
		dwShaderFlags , NULL, &m_pEffect, &pErr ))){

			MessageBox( NULL, (LPCTSTR)pErr->GetBufferPointer()
				, "ERROR", MB_OK);
	}
	else
	{
		m_hmWVP = m_pEffect->GetParameterByName( NULL, "mWVP" );
		m_hColor = m_pEffect->GetParameterByName(NULL,"Color");
		m_hTechnique = m_pEffect->GetTechniqueByName( "TShader" );
	}

	m_pEffect->SetTechnique(m_hTechnique);
}

void D3DDrawUtil::ReleaseEffect()
{
	if(m_pEffect != NULL)
	{
		m_pEffect->Release();
		m_pEffect = NULL;
	}
}

void D3DDrawUtil::CreateGeometry()
{
	CreateWireBox();
	CreateWireSphere(10, 10, D3DCOLOR_COLORVALUE(1.0f, 1.0f, 1.0f, 1.0f));
	CreateWireHalfSphere(10, 10, D3DCOLOR_COLORVALUE(0.0f, 0.0f, 0.0f, 1.0f));
	CreateWireCylinder(10, D3DCOLOR_COLORVALUE(0.0f,0.0f,0.0f,1.0f));
	CreateWireCylinderClose(10,  D3DCOLOR_COLORVALUE(0.2f, 0.2f, 0.2f, 1.0f));
}

void D3DDrawUtil::ReleaseGeometry()
{	
	if(m_pWireBoxVertex)
	{
		m_pWireBoxVertex->Release();
		m_pWireBoxVertex = NULL;
	}

	if(m_pWireBoxIndex)
	{
		m_pWireBoxIndex->Release();
		m_pWireBoxIndex = NULL;
	}

	if(m_pWireHalfSphereVertex)
	{
		m_pWireHalfSphereVertex->Release();
		m_pWireHalfSphereVertex = NULL;
	}

	if(m_pWireCylinderVertex)
	{
		m_pWireCylinderVertex->Release();
		m_pWireCylinderVertex = NULL;
	}

	if(m_pWireCylinderCloseVertex)
	{
		m_pWireCylinderCloseVertex->Release();
		m_pWireCylinderCloseVertex = NULL;
	}

	if(m_pWireSphereVertex)
	{
		m_pWireSphereVertex->Release();
		m_pWireSphereVertex = NULL;
	}
}

void D3DDrawUtil::CreateWireBox()
{
	int vertListSize;
	int vertexNumber;
	MYLINEVERTEX *vertex;
	vertexNumber=8;
    DWORD color;

    color = D3DCOLOR_COLORVALUE(0.0f, 0.0f, 0.0f, 1.0f);
	vertex = new MYLINEVERTEX[vertexNumber];
    vertListSize = vertexNumber*sizeof(MYLINEVERTEX);
	
	m_pd3dDevice->CreateVertexBuffer(vertListSize, 0,D3DFVF_XYZ | D3DFVF_DIFFUSE , 
											D3DPOOL_MANAGED, &m_pWireBoxVertex, NULL);

    vertex[0].pos.x = -1.0f; vertex[0].pos.y = 1.0f; vertex[0].pos.z = -1.0f; vertex[0].diffuse = color;
    vertex[1].pos.x = -1.0f; vertex[1].pos.y = -1.0f; vertex[1].pos.z = -1.0f; vertex[1].diffuse = color;
    vertex[2].pos.x = 1.0f; vertex[2].pos.y = -1.0f; vertex[2].pos.z = -1.0f; vertex[2].diffuse = color;
    vertex[3].pos.x = 1.0f; vertex[3].pos.y = 1.0f; vertex[3].pos.z = -1.0f; vertex[3].diffuse = color;
    
	
    vertex[4].pos.x = -1.0f; vertex[4].pos.y = 1.0f; vertex[4].pos.z = 1.0f; vertex[4].diffuse = color;
    vertex[5].pos.x = -1.0f; vertex[5].pos.y = -1.0f; vertex[5].pos.z = 1.0f; vertex[5].diffuse = color;
    vertex[6].pos.x = 1.0f; vertex[6].pos.y = -1.0f; vertex[6].pos.z = 1.0f; vertex[6].diffuse = color;
    vertex[7].pos.x = 1.0f; vertex[7].pos.y = 1.0f; vertex[7].pos.z = 1.0f; vertex[7].diffuse = color;
    
	VOID *verts = NULL; 
    
	m_pWireBoxVertex->Lock(0, 0, (void**)&verts, NULL);
	memcpy(verts, vertex, vertListSize);
	m_pWireBoxVertex->Unlock();
	
	delete[] vertex;
         
    m_pd3dDevice->CreateIndexBuffer(12*sizeof(unsigned short)*2, 0, D3DFMT_INDEX16, 
										   D3DPOOL_MANAGED, &m_pWireBoxIndex, NULL);
    	
	unsigned short index[24]={0,1,1,2,2,3,3,0,4,5,5,6,6,7,7,4,0,4,1,5,2,6,3,7};
	unsigned short* indices; 
    
	m_pWireBoxIndex->Lock(0, 0, (void **)&indices, NULL);
    memcpy(indices, index, sizeof(unsigned short)*24);
	m_pWireBoxIndex->Unlock();
}

void D3DDrawUtil::CreateWireHalfSphere(int slices, int stacks, DWORD color)
{
	int i,j;
	int vertexNumber = 0;
    int vertexCount;
	float thetaStep;
    float phiStep;
    MYLINEVERTEX* vertex;
	
	thetaStep = (3.141592f * 0.5f) / stacks;
    phiStep = (3.141592f * 2.0f) / slices;

	vertexNumber = (stacks-1)*slices*6 + 3*slices;
	
	m_iWireSphereVertex = vertexNumber;
	
	vertex = new MYLINEVERTEX[vertexNumber];
		
	int vertListSize = sizeof(MYLINEVERTEX) * vertexNumber;
	
	m_pd3dDevice->CreateVertexBuffer(vertListSize, 0, D3DFVF_XYZ | D3DFVF_DIFFUSE, 
											D3DPOOL_MANAGED, &m_pWireHalfSphereVertex, NULL);
 	vertexCount=0;
   
	for(i = 0;i < slices;i += 1)
	{
		for(j = 0;j < stacks;j += 1)
		{
			vertex[vertexCount].pos.x = sinf(thetaStep*j)*cosf(phiStep*i);	vertex[vertexCount].pos.y = sinf(thetaStep*j)*sinf(phiStep*i);	vertex[vertexCount].pos.z = cosf(thetaStep*j);
			vertex[vertexCount].diffuse = color;
			vertexCount += 1;

			vertex[vertexCount].pos.x = sinf(thetaStep*(j+1))*cosf(phiStep*i);	vertex[vertexCount].pos.y = sinf(thetaStep*(j+1))*sinf(phiStep*i);	vertex[vertexCount].pos.z = cosf(thetaStep*(j+1));
			vertex[vertexCount].diffuse = color;
			vertexCount += 1;

			vertex[vertexCount].pos.x = sinf(thetaStep*(j+1))*cosf(phiStep*(i+1));	vertex[vertexCount].pos.y = sinf(thetaStep*(j+1))*sinf(phiStep*(i+1));	vertex[vertexCount].pos.z = cosf(thetaStep*(j+1));
			vertex[vertexCount].diffuse = color;
			vertexCount += 1;
 	      
			if(j != 0)
			{
				vertex[vertexCount].pos.x = sinf(thetaStep*j)*cosf(phiStep*i);	vertex[vertexCount].pos.y = sinf(thetaStep*j)*sinf(phiStep*i);	vertex[vertexCount].pos.z = cosf(thetaStep*j);
				vertex[vertexCount].diffuse = color;
				vertexCount+=1;
            
				vertex[vertexCount].pos.x = sinf(thetaStep*(j+1))*cosf(phiStep*(i+1));	vertex[vertexCount].pos.y = sinf(thetaStep*(j+1))*sinf(phiStep*(i+1));	vertex[vertexCount].pos.z = cosf(thetaStep*(j+1));
				vertex[vertexCount].diffuse = color;
				vertexCount+=1;
			
				vertex[vertexCount].pos.x = sinf(thetaStep*j)*cosf(phiStep*(i+1));	vertex[vertexCount].pos.y = sinf(thetaStep*j)*sinf(phiStep*(i+1));	vertex[vertexCount].pos.z = cosf(thetaStep*j);
				vertex[vertexCount].diffuse = color;
				vertexCount+=1;
			}
		}
	}
   
	VOID *verts = NULL; 
    
	m_pWireHalfSphereVertex->Lock(0, 0, (void**)&verts, NULL);
	
	
	memcpy(verts, vertex, vertListSize);
	m_pWireHalfSphereVertex->Unlock();
	
	delete[] vertex;
}

void D3DDrawUtil::CreateWireCylinder(int slices, DWORD color)
{
	int i;
	int vertexNumber = 0;
    int vertexCount;
	float phiStep;
    MYLINEVERTEX* vertex;

    phiStep = (3.141592f * 2.0f) / slices;

	vertexNumber = slices*6;
	
	m_iWireCylinderVertex = vertexNumber;
	
	vertex = new MYLINEVERTEX[vertexNumber];
		
	int vertListSize = sizeof(MYLINEVERTEX) * vertexNumber;
	
	m_pd3dDevice->CreateVertexBuffer(vertListSize, 0, D3DFVF_XYZ | D3DFVF_DIFFUSE, 
											D3DPOOL_MANAGED, &m_pWireCylinderVertex, NULL);
 	vertexCount=0;
   
	for(i = 0;i < slices;i += 1)
	{
		vertex[vertexCount].pos.x = cosf(phiStep*i);	vertex[vertexCount].pos.y = sinf(phiStep*i);	vertex[vertexCount].pos.z = 1.0f;
		vertex[vertexCount].diffuse = color;
		vertexCount+=1;

		vertex[vertexCount].pos.x = cosf(phiStep*i);	vertex[vertexCount].pos.y = sinf(phiStep*i);	vertex[vertexCount].pos.z = -1.0f;
		vertex[vertexCount].diffuse = color;
		vertexCount+=1;

		vertex[vertexCount].pos.x = cosf(phiStep*(i+1));	vertex[vertexCount].pos.y = sinf(phiStep*(i+1));	vertex[vertexCount].pos.z = 1.0f;
		vertex[vertexCount].diffuse = color;
		vertexCount+=1;

		vertex[vertexCount].pos.x = cosf(phiStep*(i+1));	vertex[vertexCount].pos.y = sinf(phiStep*(i+1));	vertex[vertexCount].pos.z = 1.0f;
		vertex[vertexCount].diffuse = color;
		vertexCount+=1;

		vertex[vertexCount].pos.x = cosf(phiStep*i); vertex[vertexCount].pos.y = sinf(phiStep*i); vertex[vertexCount].pos.z = -1.0f;
		vertex[vertexCount].diffuse = color;
		vertexCount+=1;

		vertex[vertexCount].pos.x = cosf(phiStep*(i+1)); vertex[vertexCount].pos.y = sinf(phiStep*(i+1)); vertex[vertexCount].pos.z = -1.0f;
		vertex[vertexCount].diffuse = color;
		vertexCount+=1;

	}
   
	VOID *verts = NULL; 
    
	m_pWireCylinderVertex->Lock(0, 0, (void**)&verts, NULL);
	memcpy(verts, vertex, vertListSize);
	m_pWireCylinderVertex->Unlock();
	
	delete[] vertex;
}

void D3DDrawUtil::CreateWireCylinderClose(int slices, DWORD color)
{
	int i;
	int vertexNumber = 0;
	int vertexCount;
	float phiStep;
	MYLINEVERTEX* vertex;

	phiStep = (3.141592f * 2.0f ) / slices;

	vertexNumber = slices*12 ;

	m_iWireCylinderCloseVertex = vertexNumber;

	vertex = new MYLINEVERTEX[vertexNumber];

	int vertListSize = sizeof(MYLINEVERTEX) * vertexNumber;

	m_pd3dDevice->CreateVertexBuffer(vertListSize, 0, D3DFVF_XYZ | D3DFVF_NORMAL, 
		D3DPOOL_MANAGED, &m_pWireCylinderCloseVertex, NULL);
	vertexCount = 0;

	for(i = 0;i < slices;i += 1)
	{
		vertex[vertexCount].pos.x = cosf(phiStep*i); vertex[vertexCount].pos.y = sinf(phiStep*i); vertex[vertexCount].pos.z = 1.0f;
		vertex[vertexCount].diffuse = color;
		vertexCount+=1;

		vertex[vertexCount].pos.x = cosf(phiStep*i); vertex[vertexCount].pos.y = sinf(phiStep*i); vertex[vertexCount].pos.z = -1.0f;
		vertex[vertexCount].diffuse = color;
		vertexCount+=1;

		vertex[vertexCount].pos.x = cosf(phiStep*(i+1)); vertex[vertexCount].pos.y = sinf(phiStep*(i+1)); vertex[vertexCount].pos.z = 1.0f;
		vertex[vertexCount].diffuse = color;
		vertexCount+=1;

		vertex[vertexCount].pos.x = cosf(phiStep*(i+1)); vertex[vertexCount].pos.y = sinf(phiStep*(i+1)); vertex[vertexCount].pos.z = 1.0f;
		vertex[vertexCount].diffuse = color;
		vertexCount+=1;

		vertex[vertexCount].pos.x = cosf(phiStep*i); vertex[vertexCount].pos.y = sinf(phiStep*i); vertex[vertexCount].pos.z = -1.0f;
		vertex[vertexCount].diffuse = color;
		vertexCount+=1;

		vertex[vertexCount].pos.x = cosf(phiStep*(i+1)); vertex[vertexCount].pos.y = sinf(phiStep*(i+1)); vertex[vertexCount].pos.z = -1.0f;
		vertex[vertexCount].diffuse = color;
		vertexCount+=1;


		vertex[vertexCount].pos.x = 0.0f; vertex[vertexCount].pos.y = 0.0f; vertex[vertexCount].pos.z = 1.0f;
		vertex[vertexCount].diffuse = color;
		vertexCount+=1;

		vertex[vertexCount].pos.x = cosf(phiStep*i); vertex[vertexCount].pos.y = sinf(phiStep*i); vertex[vertexCount].pos.z = 1.0f;
		vertex[vertexCount].diffuse = color;
		vertexCount+=1;

		vertex[vertexCount].pos.x = cosf(phiStep*(i+1)); vertex[vertexCount].pos.y = sinf(phiStep*(i+1)); vertex[vertexCount].pos.z = 1.0f;
		vertex[vertexCount].diffuse = color;
		vertexCount+=1;


		vertex[vertexCount].pos.x = 0.0f; vertex[vertexCount].pos.y = 0.0f; vertex[vertexCount].pos.z = -1.0f;
		vertex[vertexCount].diffuse = color;
		vertexCount+=1;

		vertex[vertexCount].pos.x = cosf(phiStep*(i+1)); vertex[vertexCount].pos.y = sinf(phiStep*(i+1)); vertex[vertexCount].pos.z = -1.0f;
		vertex[vertexCount].diffuse = color;
		vertexCount+=1;

		vertex[vertexCount].pos.x = cosf(phiStep*i); vertex[vertexCount].pos.y = sinf(phiStep*i); vertex[vertexCount].pos.z = -1.0f;
		vertex[vertexCount].diffuse = color;
		vertexCount+=1;

	}

	VOID *verts = NULL; 

	m_pWireCylinderCloseVertex->Lock(0, 0, (void**)&verts, NULL);
	memcpy(verts, vertex, vertListSize);
	m_pWireCylinderCloseVertex->Unlock();

	delete[] vertex;
}

void D3DDrawUtil::CreateWireSphere(int slices, int stacks, DWORD color)
{
	int i,j;
	int vertexNumber = 0;
    int vertexCount;
	float thetaStep;
    float phiStep;
    MYLINEVERTEX* vertex;
	
	thetaStep = (3.141592f) / stacks;
    phiStep = (3.141592f * 2.0f) / slices;

	vertexNumber = slices*stacks*2 + slices*(stacks-1)*2;
	
	m_iWireSphereVertexNumber = vertexNumber;
	
	vertex = new MYLINEVERTEX[vertexNumber];
		
	int vertListSize = sizeof(MYLINEVERTEX) * vertexNumber;
	
	m_pd3dDevice->CreateVertexBuffer(vertListSize, 0, D3DFVF_XYZ | D3DFVF_DIFFUSE, 
											D3DPOOL_MANAGED, &m_pWireSphereVertex, NULL);
 	vertexCount=0;
   
	for(i = 0;i < slices;i += 1)
	{
		for(j = 0;j < stacks;j += 1)
		{
			vertex[vertexCount].pos.x = sinf(thetaStep*j)*cosf(phiStep*i);	vertex[vertexCount].pos.y = sinf(thetaStep*j)*sinf(phiStep*i);	vertex[vertexCount].pos.z = cosf(thetaStep*j);
			vertex[vertexCount].diffuse = color;
			vertexCount += 1;

			vertex[vertexCount].pos.x = sinf(thetaStep*(j+1))*cosf(phiStep*i);	vertex[vertexCount].pos.y = sinf(thetaStep*(j+1))*sinf(phiStep*i);	vertex[vertexCount].pos.z = cosf(thetaStep*(j+1));
			vertex[vertexCount].diffuse = color;
			vertexCount += 1;
 	     }
	}
   
	for(j = 1;j < stacks; j+= 1)
	{
		for(i = 0;i < slices;i += 1)
		{
			vertex[vertexCount].pos.x = sinf(thetaStep*j)*cosf(phiStep*i);	vertex[vertexCount].pos.y = sinf(thetaStep*j)*sinf(phiStep*i);	vertex[vertexCount].pos.z = cosf(thetaStep*j);
			vertex[vertexCount].diffuse = color;
			vertexCount += 1;

			vertex[vertexCount].pos.x = sinf(thetaStep*(j))*cosf(phiStep*(i+1));	vertex[vertexCount].pos.y = sinf(thetaStep*j)*sinf(phiStep*(i+1));	vertex[vertexCount].pos.z = cosf(thetaStep*j);
			vertex[vertexCount].diffuse = color;
			vertexCount += 1;
 	     }
	}
   	
	VOID *verts = NULL; 
    
	m_pWireSphereVertex->Lock(0, 0, (void**)&verts, NULL);
	
	
	memcpy(verts, vertex, vertListSize);
	m_pWireSphereVertex->Unlock();
	
	delete[] vertex;
}

LPDIRECT3DDEVICE9 D3DDrawUtil::GetDevice()
{
	return m_pd3dDevice;
}

void D3DDrawUtil::SetViewProjectTM(D3DXMATRIX &viewProjectTM)
{
	m_viewProjectionTM = viewProjectTM;
}

void D3DDrawUtil::DrawGrid(float xLength, float yLength, int xGrid, int yGrid, DWORD color)
{
	MYLINEVERTEX *pVertex;
	int iCount;
	float xStep, yStep;

	pVertex = new MYLINEVERTEX[2*(xGrid+1) + 2*(yGrid+1)];

	xStep = 2.0f*xLength/xGrid;
	yStep = 2.0f*yLength/yGrid;


	iCount = 0;
	for(int i = 0; i <= xGrid; i += 1)
	{
		pVertex[iCount].pos.x = -xLength + i*xStep; pVertex[iCount].pos.y = -yLength; pVertex[iCount].pos.z = 0.0f; pVertex[iCount].diffuse = color;  
		iCount += 1;

		pVertex[iCount].pos.x = -xLength + i*xStep; pVertex[iCount].pos.y = +yLength; pVertex[iCount].pos.z = 0.0f; pVertex[iCount].diffuse = color;  
		iCount += 1;
	}

	for(int i = 0; i <= yGrid; i += 1)
	{
		pVertex[iCount].pos.x = -xLength; pVertex[iCount].pos.y = -yLength + i*yStep; pVertex[iCount].pos.z = 0.0f; pVertex[iCount].diffuse = color;  
		iCount += 1;

		pVertex[iCount].pos.x = xLength; pVertex[iCount].pos.y = -yLength + i*yStep; pVertex[iCount].pos.z = 0.0f; pVertex[iCount].diffuse = color;  
		iCount += 1;
	}

	m_pd3dDevice->SetRenderState(D3DRS_ALPHABLENDENABLE, true); 
	m_pd3dDevice->SetRenderState(D3DRS_LIGHTING, false);
	m_pd3dDevice->SetTransform(D3DTS_WORLD, m_pMatrixStack->GetTop());
	m_pd3dDevice->SetFVF(MYLINEVERTEX::GetFVF());
	m_pd3dDevice->DrawPrimitiveUP(D3DPT_LINELIST, iCount/2, pVertex, sizeof(MYLINEVERTEX)); 
	m_pd3dDevice->SetRenderState(D3DRS_ALPHABLENDENABLE, false); 

	delete[] pVertex;
}

void D3DDrawUtil::DrawWireSphere(float radius, DWORD color)
{
	D3DXMATRIX worldTM, worldViewProjectTM, invWorldTM;
	D3DXVECTOR4 vColor;
	
	m_pMatrixStack->Push();
	m_pMatrixStack->ScaleLocal(radius, radius, radius);
	worldTM = *(m_pMatrixStack->GetTop());
	m_pMatrixStack->Pop();
	worldViewProjectTM = worldTM * m_viewProjectionTM ;
	
	vColor.w = (float)(color >> 24); 
	vColor.x = (float)((color << 8) >> 24);
	vColor.y = (float)((color << 16) >> 24);
	vColor.z = (float)((color << 24) >> 24);
	vColor /= 255.0f;

	m_pEffect->SetVector(m_hColor, &vColor);
	m_pEffect->SetMatrix(m_hmWVP,&worldViewProjectTM);	

	m_pEffect->Begin( NULL, 0 );
	m_pEffect->BeginPass( 0 );

	m_pd3dDevice->SetFVF(MYLINEVERTEX::GetFVF());
	m_pd3dDevice->SetStreamSource( 0, m_pWireSphereVertex, 0, sizeof(MYLINEVERTEX) );
	m_pd3dDevice->DrawPrimitive( D3DPT_LINELIST, 0, m_iWireSphereVertexNumber/2);

	m_pEffect->EndPass();  
	m_pEffect->End(); 
}

void D3DDrawUtil::DrawWireHalfSphere(float radius, DWORD color)
{
	m_pd3dDevice->SetRenderState(D3DRS_FILLMODE, D3DFILL_WIREFRAME);

	D3DXMATRIX worldViewProjectTM;
	D3DXMATRIX worldTM;
	D3DXVECTOR4 vColor;

	vColor.w = (float)(color >> 24); 
	vColor.x = (float)((color << 8) >> 24);
	vColor.y = (float)((color << 16) >> 24);
	vColor.z = (float)((color << 24) >> 24);
	vColor /= 255.0f;
	m_pEffect->SetVector(m_hColor, &vColor);

	m_pMatrixStack->Push();
	m_pMatrixStack->ScaleLocal(radius, radius, radius);
	worldTM = *(m_pMatrixStack->GetTop());
	m_pMatrixStack->Pop();
	worldViewProjectTM = worldTM * m_viewProjectionTM ;
	
	m_pd3dDevice->SetFVF(MYLINEVERTEX::GetFVF());
	
	m_pEffect->SetMatrix(m_hmWVP,&worldViewProjectTM);	

	m_pEffect->Begin( NULL, 0 );
	m_pEffect->BeginPass( 0 );

	m_pd3dDevice->SetStreamSource( 0, m_pWireHalfSphereVertex, 0, sizeof(MYLINEVERTEX) );
	m_pd3dDevice->DrawPrimitive( D3DPT_TRIANGLELIST, 0, m_iWireSphereVertex/3);

	m_pEffect->EndPass();  
	m_pEffect->End();

	m_pd3dDevice->SetRenderState(D3DRS_FILLMODE, D3DFILL_SOLID);
}

void D3DDrawUtil::DrawWireBox(D3DXVECTOR3 *vDemension, DWORD color)
{
	D3DXMATRIX worldViewProjectTM;
	D3DXMATRIX worldTM;
	
	m_pMatrixStack->Push();
	m_pMatrixStack->ScaleLocal(vDemension->x, vDemension->y, vDemension->z);
	worldTM = *(m_pMatrixStack->GetTop());
	m_pMatrixStack->Pop();

	worldViewProjectTM = worldTM * m_viewProjectionTM;
	m_pEffect->SetMatrix(m_hmWVP,&worldViewProjectTM);	

	D3DXVECTOR4 vColor;

	vColor.w = (float)(color >> 24); 
	vColor.x = (float)((color << 8) >> 24);
	vColor.y = (float)((color << 16) >> 24);
	vColor.z = (float)((color << 24) >> 24);
	vColor /= 255.0f;
	
	m_pEffect->SetVector(m_hColor, &vColor);

	m_pEffect->Begin( NULL, 0 );
	m_pEffect->BeginPass( 0 );

	m_pd3dDevice->SetFVF(MYLINEVERTEX::GetFVF());
	m_pd3dDevice->SetStreamSource(0, m_pWireBoxVertex, 0, sizeof(MYLINEVERTEX));
	m_pd3dDevice->SetIndices(m_pWireBoxIndex);
	m_pd3dDevice->DrawIndexedPrimitive(D3DPT_LINELIST, 0, 0, 8, 0, 12);

	m_pEffect->EndPass();  
	m_pEffect->End(); 
}

void D3DDrawUtil::DrawWireCylinderClose(float fHeight, float fRadius, DWORD color)
{
	m_pd3dDevice->SetRenderState(D3DRS_FILLMODE, D3DFILL_WIREFRAME);

	D3DXMATRIX worldViewProjectTM;
	D3DXMATRIX worldTM;
	D3DXVECTOR4 vColor;

	vColor.w = (float)(color >> 24); 
	vColor.x = (float)((color << 8) >> 24);
	vColor.y = (float)((color << 16) >> 24);
	vColor.z = (float)((color << 24) >> 24);
	vColor /= 255.0f;
	m_pEffect->SetVector(m_hColor, &vColor);

	m_pMatrixStack->Push();
	m_pMatrixStack->ScaleLocal(fRadius, fRadius, fHeight);
	worldTM = *(m_pMatrixStack->GetTop());
	m_pMatrixStack->Pop();
	
	worldViewProjectTM = worldTM * m_viewProjectionTM;
	m_pEffect->SetMatrix(m_hmWVP,&worldViewProjectTM);	

	m_pd3dDevice->SetFVF(MYLINEVERTEX::GetFVF());

	m_pEffect->Begin( NULL, 0 );
	m_pEffect->BeginPass( 0 );

	m_pd3dDevice->SetStreamSource( 0, m_pWireCylinderCloseVertex, 0, sizeof(MYLINEVERTEX) );
	m_pd3dDevice->DrawPrimitive( D3DPT_TRIANGLELIST, 0, m_iWireCylinderCloseVertex/3);

	m_pEffect->EndPass();  
	m_pEffect->End(); 

	m_pd3dDevice->SetRenderState(D3DRS_FILLMODE, D3DFILL_SOLID);
}

void D3DDrawUtil::DrawWireCylinder(float fHeight, float fRadius, DWORD color)
{
	m_pd3dDevice->SetRenderState(D3DRS_FILLMODE, D3DFILL_WIREFRAME);

	D3DXMATRIX worldViewProjectTM;
	D3DXMATRIX worldTM;
	D3DXVECTOR4 vColor;
	
	vColor.w = (float)(color >> 24); 
	vColor.x = (float)((color << 8) >> 24);
	vColor.y = (float)((color << 16) >> 24);
	vColor.z = (float)((color << 24) >> 24);
	vColor /= 255.0f;
	m_pEffect->SetVector(m_hColor, &vColor);

	m_pMatrixStack->Push();
	m_pMatrixStack->ScaleLocal(fRadius, fRadius, fHeight);
	worldTM = *(m_pMatrixStack->GetTop());
	m_pMatrixStack->Pop();

	worldViewProjectTM = worldTM * m_viewProjectionTM;
	m_pEffect->SetMatrix(m_hmWVP,&worldViewProjectTM);	
	
	m_pd3dDevice->SetFVF(MYLINEVERTEX::GetFVF());
	
	m_pEffect->Begin( NULL, 0 );
	m_pEffect->BeginPass( 0 );

	m_pd3dDevice->SetStreamSource( 0, m_pWireCylinderVertex, 0, sizeof(MYLINEVERTEX) );
	m_pd3dDevice->DrawPrimitive( D3DPT_TRIANGLELIST, 0, m_iWireCylinderVertex/3);

	m_pEffect->EndPass();  
	m_pEffect->End();

	m_pd3dDevice->SetRenderState(D3DRS_FILLMODE, D3DFILL_SOLID);
}

void D3DDrawUtil::DrawWireCapsule(float fHeight, float fRadius, DWORD color)
{
	m_pMatrixStack->Push();
	m_pMatrixStack->TranslateLocal(0.0f, 0.0f, fHeight);
	DrawWireHalfSphere(fRadius, color);  
	m_pMatrixStack->Pop();

	m_pMatrixStack->Push();
	DrawWireCylinder(fHeight, fRadius, color);
	m_pMatrixStack->Pop();

	m_pMatrixStack->Push();
	m_pMatrixStack->RotateAxisLocal(&D3DXVECTOR3(0.0f, 1.0f, 0.0f), D3DX_PI);
	m_pMatrixStack->TranslateLocal(0.0f, 0.0f, fHeight);
	DrawWireHalfSphere(fRadius, color);    
	m_pMatrixStack->Pop();
}

void D3DDrawUtil::DrawWireRigidBody(btRigidBody *pBody, DWORD color)
{
	if(!pBody)
		return;
	
	btTransform _worldTM;
	pBody->getMotionState()->getWorldTransform(_worldTM);
	D3DXMATRIX worldM = btTransformToD3DXMATIRX(_worldTM);

	m_pMatrixStack->Push();
	m_pMatrixStack->MultMatrixLocal(&worldM);

	DrawWireShape(pBody->getCollisionShape(), color);

	m_pMatrixStack->Pop();
}

void D3DDrawUtil::DrawWireShape(const btCollisionShape* pShape, DWORD color)
{
	switch(pShape->getShapeType())
	{
	case BOX_SHAPE_PROXYTYPE:
		{
			DrawWireShapeBox((btBoxShape*)pShape, color);
			break;
		}

	case SPHERE_SHAPE_PROXYTYPE:
		{
			DrawWireShapeSphere((btSphereShape*)pShape, color);
			break;
		}

	case CAPSULE_SHAPE_PROXYTYPE:
		{
			DrawWireShapeCapsule((btCapsuleShape*)pShape, color);
			break;
		}

	case CYLINDER_SHAPE_PROXYTYPE:
		{
			DrawWireShapeCylinder((btCylinderShape*)pShape, color);
			break;
		}

	case  COMPOUND_SHAPE_PROXYTYPE:
		{
			btCompoundShape *pCompoundShape = (btCompoundShape*)pShape;
			int iShapeNumber = pCompoundShape->getNumChildShapes();

			for(int i=0; i<iShapeNumber; i++)
			{
				btCollisionShape *pChildShape = pCompoundShape->getChildShape(i);
				D3DXMATRIX transformM = btTransformToD3DXMATIRX(pCompoundShape->getChildTransform(i));

				m_pMatrixStack->Push();
				m_pMatrixStack->MultMatrixLocal(&transformM);
				DrawWireShape(pChildShape, color);
				m_pMatrixStack->Pop();
			}

			break;
		}
	}
}

void D3DDrawUtil::DrawWireShapeBox(const btBoxShape* pShape, DWORD color)
{
	btVector3 dim = pShape->getHalfExtentsWithMargin();
	DrawWireBox(&D3DXVECTOR3(dim.m_floats[0], dim.m_floats[1], dim.m_floats[2]), color);
}

void D3DDrawUtil::DrawWireShapeSphere(const btSphereShape* pShape, DWORD color)
{
	float fRadius = pShape->getRadius();
	DrawWireSphere(fRadius, color);
}

void D3DDrawUtil::DrawWireShapeCapsule(const btCapsuleShape* pShape, DWORD color)
{
	DrawWireCapsule(pShape->getHalfHeight(), pShape->getRadius(), color);
}

void D3DDrawUtil::DrawWireShapeCylinder(const btCylinderShape* pShape, DWORD color)
{
	btVector3 dim = pShape->getHalfExtentsWithMargin();
	DrawWireCylinderClose(dim.getZ(), dim.getX(), color);
}



