#ifndef  _D3D_DRAWUTIL_H
#define  _D3D_DRAWUTIL_H

#include<d3d9.h>
#include<d3dx9.h>
#include<Render/DataType.h>
#include<btBulletDynamicsCommon.h>

class D3DDrawUtil
{
public:
	D3DDrawUtil(HWND hWnd);
	virtual ~D3DDrawUtil(void);

protected:
	static D3DDrawUtil *m_pInstance;

	HRESULT CreateD3D(HWND hWnd);
	void ReleaseD3D();
	void CreateMatrixStack();
	void ReleaseMatrixStack();
	void CreateFont();
	void ReleaseFont();
	void CreateEffect();
	void ReleaseEffect();
	void CreateGeometry();
	void ReleaseGeometry();
	
	void CreateWireBox();
	void CreateWireHalfSphere(int slices, int stacks, DWORD color);
	void CreateWireCylinder(int slices, DWORD color);
	void CreateWireCylinderClose(int slices, DWORD color);
	void CreateWireSphere(int slices, int stacks, DWORD color);

public:
	static void CreateInstance(HWND hWnd);
	static D3DDrawUtil* Instance();
	static void DestoryInstance();

	LPDIRECT3DDEVICE9 GetDevice();
	
	void SetViewProjectTM(D3DXMATRIX &viewProjectTM);
		
	void DrawAxis(float size);
	void DrawGrid(float xLength, float yLength, int xGrid, int yGrid, DWORD color);
	void DrawWireSphere(float radius, DWORD color);
	void DrawWireHalfSphere(float radius, DWORD color);
	void DrawWireBox(D3DXVECTOR3 *vDemension, DWORD color);
	void DrawWireCylinderClose(float fHeight, float fRadius, DWORD color);
	void DrawWireCylinder(float fHeight, float fRadius, DWORD color);
	void DrawWireCapsule(float fHeight, float fRadius, DWORD color);
	
	void DrawWireRigidBody(btRigidBody *pBody, DWORD color);
	void DrawWireShape(const btCollisionShape* pShape, DWORD color);
	void DrawWireShapeBox(const btBoxShape* pShape, DWORD color);
	void DrawWireShapeSphere(const btSphereShape* pShape, DWORD color);
	void DrawWireShapeCapsule(const btCapsuleShape* pShape, DWORD color);
	void DrawWireShapeCylinder(const btCylinderShape* pShape, DWORD color);
	
private:
	LPDIRECT3DDEVICE9	m_pd3dDevice;
    LPD3DXMATRIXSTACK	m_pMatrixStack;
	LPD3DXFONT			m_pFont;
	LPDIRECT3D9			m_pD3D;

	LPD3DXEFFECT m_pEffect;
	D3DXHANDLE m_hTechnique;
	D3DXHANDLE m_hmWVP;
	D3DXHANDLE m_hColor;
	D3DXMATRIX m_viewProjectionTM;
	
	LPDIRECT3DVERTEXBUFFER9 m_pWireBoxVertex; 
    LPDIRECT3DINDEXBUFFER9 m_pWireBoxIndex;	
	
	LPDIRECT3DVERTEXBUFFER9 m_pWireHalfSphereVertex; 
    int m_iWireSphereVertex;

	LPDIRECT3DVERTEXBUFFER9 m_pWireCylinderVertex;
	int m_iWireCylinderVertex;

	LPDIRECT3DVERTEXBUFFER9 m_pHalfCylinderCloseVertex;
	int m_iHalfCylinderCloseVertex;

	LPDIRECT3DVERTEXBUFFER9 m_pWireCylinderCloseVertex;
	int m_iWireCylinderCloseVertex;
	
	LPDIRECT3DVERTEXBUFFER9 m_pWireSphereVertex;
	int m_iWireSphereVertexNumber;
};

#endif

